#ifndef lint
/* static char sccsid[] = "%Z%%M% %I% %E% 1993 by Robin Miller"; */
#endif
/****************************************************************************
 *									    *
 *			  COPYRIGHT (c) 1990 - 1995			    *
 *			   This Software Provided			    *
 *				     By					    *
 *			   MILLER W J & ASSOCIATES			    *
 *			     108 1/2 Dracut Road			    *
 *			      Hudson, NH 03051				    *
 *			       (603) 883-2355				    *
 *									    *
 * Permission to use, copy, modify, distribute and sell this software and   *
 * its documentation for any purpose and without fee is hereby granted,	    *
 * provided that the above copyright notice appear in all copies and that   *
 * both that copyright notice and this permission notice appear in the	    *
 * supporting documentation, and that the name of the author not be used    *
 * in advertising or publicity pertaining to distribution of the software   *
 * without specific, written prior permission.				    *
 *									    *
 * THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, 	    *
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN	    *
 * NO EVENT SHALL HE BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL   *
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR    *
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS  *
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF   *
 * THIS SOFTWARE.							    *
 *									    *
 ****************************************************************************/
/*
 * Module:	dttape.c
 * Author:	Robin T. Miller
 * Date:	August 20, 1993
 *
 * Description:
 *	This file contains the tape support functions.
 *
 * Modification History:
 *	Add DoWriteFileMark() function for QNX 4.22 Operating System.
 *
 */

#include "dt.h"

#if defined(_QNX_SOURCE)
#  include <sys/qioctl.h>
#else /* !defined(_QNX_SOURCE) */
#  include <sys/ioctl.h>
#  include <sys/mtio.h>
#endif /* defined(_QNX_SOURCE) */

#if !defined(_QNX_SOURCE)
/************************************************************************
 *									*
 * DoMtOp()	Setup & Do a Magtape Operation.				*
 *									*
 * Inputs:	fd = The file descriptor.				*
 *		cmd = The magtape command to issue.			*
 *		count = The command count (if any).			*
 *		msg = The error message for failures.			*
 *									*
 * Return Value:							*
 *		Returns 0 / -1 = SUCCESS / FAILURE.			*
 *									*
 ************************************************************************/
int
DoMtOp (int fd, short cmd, daddr_t count, caddr_t msgp)
{
	struct mtop Mtop;
	struct mtop *mtop = &Mtop;

	mtop->mt_op = cmd;

	/*
	 * Setup the 'mt' command count (if any).
	 */
	if ((mtop->mt_count = count) < 0) {
	    mtop->mt_count = 1;
	}

	if (debug_flag) {
	    Fprintf ("Issuing '%s' command, count = %d...\n",
					msgp,  mtop->mt_count);
	}

	return (DoIoctl (fd, MTIOCTOP, (caddr_t) mtop, msgp));
}

/************************************************************************
 *									*
 * DoXXXXX()	Setup & Do Specific Magtape Operations.			*
 *									*
 * Description:								*
 *	These functions provide a simplistic interface for issuing	*
 * magtape commands from within the program.  They all take 'count'	*
 * as an argument, except for those which do not take a count.		*
 *									*
 * Inputs:	fd = The file descriptor.				*
 *		count = The command count (if any).			*
 *									*
 * Return Value:							*
 *		Returns 0 / -1 = SUCCESS / FAILURE.			*
 *									*
 ************************************************************************/
int
DoWriteFileMark (int fd, daddr_t count)
{
	short cmd = MTWEOF;
	return (DoMtOp (fd, cmd, count, "write file mark"));
}

int
DoForwardSpaceFile (int fd, daddr_t count)
{
	short cmd = MTFSF;
	return (DoMtOp (fd, cmd, count, "forward space file"));
}

int
DoBackwardSpaceFile (int fd, daddr_t count)
{
	short cmd = MTBSF;
	return (DoMtOp (fd, cmd, count, "backward space file"));
}

int
DoForwardSpaceRecord (int fd, daddr_t count)
{
	short cmd = MTFSR;
	return (DoMtOp (fd, cmd, count, "forward space record"));
}

int
DoBackwardSpaceRecord (int fd, daddr_t count)
{
	short cmd = MTBSR;
	return (DoMtOp (fd, cmd, count, "backward space record"));
}

int
DoRewindTape (int fd)
{
	short cmd = MTREW;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "rewind tape"));
}

int
DoTapeOffline (int fd)
{
	short cmd = MTOFFL;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "tape offline"));
}

#if !defined(OSFMK)
int
DoRetensionTape (int fd)
{
	short cmd = MTRETEN;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "retension tape"));
}
#endif /* !defined(OSFMK) */

#if defined(__osf__)			/* Really DEC specific. */

int
DoSpaceEndOfData (int fd)
{
	short cmd = MTSEOD;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "space to end of data"));
}

int
DoEraseTape (int fd)
{
	short cmd = MTERASE;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "erase tape"));
}

int
DoTapeOnline (int fd)
{
	short cmd = MTONLINE;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "tape online"));
}

int
DoLoadTape (int fd)
{
	short cmd = MTLOAD;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "load tape"));
}

int
DoUnloadTape (int fd)
{
	short cmd = MTUNLOAD;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "unload tape"));
}

#endif /* defined(__osf__) */

#if defined(sun)

int
DoSpaceEndOfData (int fd)
{
	short cmd = MTEOM;
	return (DoMtOp (fd, cmd, (daddr_t) 0, "space to end of data"));
}

#endif /* defined(sun) */

/************************************************************************
 *									*
 * DoIoctl()	Do An I/O Control Command.				*
 *									*
 * Description:								*
 *	This function issues the specified I/O control command to the	*
 * device driver.							*
 *									*
 * Inputs:	fd = The file descriptor.				*
 *		cmd = The I/O control command.				*
 *		argp = The command argument to pass.			*
 *		msgp = The message to display on errors.		*
 *									*
 * Return Value:							*
 *		Returns 0 / -1 = SUCCESS / FAILURE.			*
 *									*
 ************************************************************************/
int
DoIoctl (int fd, int cmd, caddr_t argp, caddr_t msgp)
{
	int status;

	if ((status = ioctl (fd, cmd, argp)) < 0) {
	    perror (msgp);
	    if (RecordError() >= error_limit) {
		terminate (FATAL_ERROR);
	    }
	}
	return (status);
}

#else /* defined(_QNX_SOURCE) */
/************************************************************************
 *									*
 * DoXXXXX()	Setup & Do Specific Magtape Operations.			*
 *									*
 * Description:								*
 *	These functions provide a simplistic interface for issuing	*
 * magtape commands from within the program.  They all take 'count'	*
 * as an argument, except for those which do not take a count.		*
 *									*
 * Inputs:	fd = The file descriptor.				*
 *		count = The command count (if any).			*
 *									*
 * Return Value:							*
 *		Returns 0 / -1 = SUCCESS / FAILURE.			*
 *									*
 ************************************************************************/
int
DoWriteFileMark (int fd, daddr_t count)
{
	char *msgp = "write file mark";
	QIC02_MSG_STRUCT qic02ms;
	qic02ms.Header.Command = T_WRITE_FM;

	if (debug_flag) {
	    Fprintf ("Issuing '%s' command, count = %d...\n", msgp, count);
	}
	while (count--) {
	    if (qnx_ioctl (fd,
		       QCTL_RAW_CMD,
		       &qic02ms, sizeof (QIC02_HEADER_STRUCT),
		       &qic02ms, sizeof (QIC02_MSG_STRUCT)) == -1) {
		perror (msgp);
		if (RecordError() >= error_limit) {
		    terminate (FATAL_ERROR);
		    return (FAILURE);
		}
	    }
	}
	return (SUCCESS);
}
#endif /* !defined(_QNX_SOURCE) */
